<?php

namespace Drupal\commerce_payment_mkb\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsAuthorizationsInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsRefundsInterface;
use Drupal\commerce_price\Price;
use Drupal\Core\Form\FormStateInterface;
use Mkb\Sdk\Client;
use Mkb\Sdk\Signature;
use Mkb\Sdk\Status;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the MKB offsite Checkout payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "mkb_redirect_checkout",
 *   label = @Translation("МКБ Интернет-эквайринг"),
 *   display_label = @Translation("МКБ Интернет-эквайринг"),
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_payment_mkb\PluginForm\OffsiteRedirect\PaymentOffsiteForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "mastercard", "visa", "mir"
 *   },
 * )
 */
class OffsiteRedirect extends OffsitePaymentGatewayBase implements SupportsAuthorizationsInterface, SupportsRefundsInterface
{

	/**
	 * {@inheritdoc}
	 */
	public function defaultConfiguration()
	{
		return [
			'mid'           => '',
			'secret'        => '',
			'merchant_mail' => '',
			'login'         => '',
			'password'      => '',
		] + parent::defaultConfiguration();
	}

	/**
	 * {@inheritdoc}
	 */
	public function buildConfigurationForm(array $form, FormStateInterface $form_state)
	{
		$form = parent::buildConfigurationForm($form, $form_state);

		$form['mid'] = [
			'#type'          => 'textfield',
			'#title'         => 'Идентификатор Мерчанта',
			'#default_value' => $this->configuration['mid'],
			'#required'      => true,
		];

		$form['secret'] = [
			'#type'          => 'textfield',
			'#title'         => 'Пароль мерчанта',
			'#default_value' => $this->configuration['secret'],
		];

		$form['login'] = [
			'#type'          => 'textfield',
			'#title'         => 'Логин',
			'#default_value' => $this->configuration['login'],
		];

		$form['password'] = [
			'#type'          => 'textfield',
			'#title'         => 'Пароль',
			'#default_value' => $this->configuration['password'],
		];

		$form['merchant_mail'] = [
			'#type'          => 'textfield',
			'#title'         => 'Email магазина',
			'#default_value' => $this->configuration['merchant_mail'],
		];

		return $form;
	}

	/**
	 * {@inheritdoc}
	 */
	public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
	{
		parent::submitConfigurationForm($form, $form_state);
		if (!$form_state->getErrors()) {
			$values                               = $form_state->getValue($form['#parents']);
			$this->configuration['mid']           = $values['mid'];
			$this->configuration['secret']        = $values['secret'];
			$this->configuration['merchant_mail'] = $values['merchant_mail'];
			$this->configuration['login']         = $values['login'];
			$this->configuration['password']      = $values['password'];
		}
	}

	/**
	 * 	 * URL: http://site.domain/payment/notify/mkb
	 *
	 * {@inheritdoc}
	 *
	 * @param Request $request
	 * @return null|void
	 */
	public function onNotify(Request $request)
	{
		$transaction_id = $request->request->get('OrderID');
		$order_storage  = $this->entityTypeManager->getStorage('commerce_order');
		/** @var $order OrderInterface */
		$order = $order_storage->load($transaction_id);
		if (is_null($order)) {
			throw new \Exception('Order not found');
		}
		$post      = $request->request->all();
		$signature = Signature::calcPaymentNotifySign($post, $this->configuration['secret']);
		if (strcmp($request->request->get('Signature'), $signature) !== 0) {
			throw new \Exception('Wrong Signature');
		}
		if ($post['ResponseCode'] !== '1') {
			return;
		}

		$info   = $this->getTransactionInfo($transaction_id);
		$status = substr($info['status'], 0, 1);

		$state = $status == Status::PREAUTH ? 'authorization' : 'completed';

		$order->set('state', 'completed');
		$order->save();
		$payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
		$payment         = $payment_storage->create([
			'state'           => $state,
			'amount'          => $order->getTotalPrice(),
			'payment_gateway' => $this->entityId,
			'order_id'        => $order->id(),
			'remote_id'       => $request->request->get('AuthCode'),
			'remote_state'    => $info['status'],
		]);

		$payment->save();
	}

	public function capturePayment(PaymentInterface $payment, Price $amount = null)
	{
		$this->assertPaymentState($payment, ['authorization']);
		// If not specified, capture the entire amount.
		$data = $this->prepareActionData($payment);
		$client = new Client();
		if ($this->configuration['mode'] === 'test') {
			$client->setTestMode();
		}
		try {
			$data['Signature'] = Signature::calcFinActionSign($data, $this->configuration['secret']);
			$result = $client->capturePayment($data);
		} catch (\Exception $e){
			throw new PaymentGatewayException($e->getMessage());
		}

		$responseCode = $result['ResponseCode'] ?? 0;
		if ($responseCode != 1) {
			throw new PaymentGatewayException($result['ReasonCodeDesc']);
		}

		$payment->setState('completed');
		$payment->save();
	}

	public function voidPayment(PaymentInterface $payment)
	{
		$this->assertPaymentState($payment, ['authorization']);
		// If not specified, refund the entire amount.
		$data = $this->prepareActionData($payment);
		$client = new Client();
		if ($this->configuration['mode'] === 'test') {
			$client->setTestMode();
		}
		try {
			$data['Signature'] = Signature::calcFinActionSign($data, $this->configuration['secret']);
			$result = $client->reversePayment($data);
		} catch (\Exception $e){
			throw new PaymentGatewayException($e->getMessage());
		}

		$responseCode = $result['ResponseCode'] ?? 0;
		if ($responseCode != 1) {
			throw new PaymentGatewayException($result['ReasonCodeDesc']);
		}

		$payment->setState('authorization_voided');
		$payment->save();
	}

	public function refundPayment(PaymentInterface $payment, Price $amount = null)
	{
		$this->assertPaymentState($payment, ['completed']);
		// If not specified, refund the entire amount.
		$data = $this->prepareActionData($payment);
		$client = new Client();
		if ($this->configuration['mode'] === 'test') {
			$client->setTestMode();
		}
		try {
			$data['Signature'] = Signature::calcFinActionSign($data, $this->configuration['secret']);
			$result = $client->refundPayment($data);
		} catch (\Exception $e){
			throw new PaymentGatewayException($e->getMessage());
		}

		$responseCode = $result['ResponseCode'] ?? 0;
		if ($responseCode != 1) {
			throw new PaymentGatewayException($result['ReasonCodeDesc']);
		}

		$payment->setState('refunded');
		$payment->setRefundedAmount($payment->getAmount());
		$payment->save();
	}

	private function getTransactionInfo($order_id)
	{
		$client = new Client();
		if ($this->configuration['mode'] === 'test') {
			$client->setTestMode();
		}

		$data = [
			'login'    => $this->configuration['login'],
			'password' => $this->configuration['password'],
			'Status'   => 'Full',
			'OrderID'  => $order_id,
			'MerID'    => $this->configuration['mid'],
			'Type'     => 'Purchase',
		];

		$result = $client->getOrderStatus($data);
		return $result;
	}

	private function prepareActionData(PaymentInterface $payment)
	{
		$amount = $payment->getAmount()->getNumber() * 100;
		$amount = sprintf('%012d', $amount);

		$query   = [
			'MerID'               => $this->configuration['mid'],
			'AcqID'               => 443222,
			'PurchaseAmt'         => $amount,
			'OrderID'             => $payment->getOrder()->id(),
			'AuthorizationNumber' => substr($payment->getRemoteId(), 0, 6),
			'Amount'              => $amount,
		];
		return $query;
	}
}
