<?php

namespace Mkb\Sdk;

use Mkb\Sdk\Exceptions\EmptyPropertyException;
use Mkb\Sdk\Exceptions\ExtensionNotFoundException;
use Mkb\Sdk\Exceptions\ServletResponseException;

class Client
{
	const PROD_FORM_URL = 'https://mpi.mkb.ru/';
	const TEST_FORM_URL = 'https://mpi.mkb.ru:9443/';
	const PROD_URL      = 'https://mpi.mkb.ru:8443/';
	const TEST_URL      = 'https://mpi.mkb.ru:9443/';

	private $testMode = 0;

	public function setTestMode()
	{
		$this->testMode = 1;
	}

	public function getOrderStatus($data)
	{
		$result = $this->request($data, 'finoperate/dogetorderstatusservlet');
		if (!$result) {
			throw new ServletResponseException('Empty order status servlet response');
		}
		$xml = simplexml_load_string($result);
		if (!$xml) {
			throw new ServletResponseException('Order status servlet response is not xml');
		}
		$data = [];
		foreach ($xml as $key => $value) {
			$data[(string) $key] = (string) $value;
		}
		if (!empty($data['error'])) {
			throw new ServletResponseException($data['error']);
		}
		return $data;
	}

	public function getReceipt($data)
	{
		return $this->request($data, 'finoperate1/getReceipt');
	}

	public function removeClient($data)
	{
		return $this->request($data, "eCom_api/client_identification/client/{$data['mid']}/{$data['client_id']}", 'DELETE');
	}

	public function removeClientCard($data)
	{
		return $this->request($data, "eCom_api/client_identification/card/{$data['mid']}/{$data['client_id']}/{$data['card_id']}", 'DELETE');
	}

	public function removeAllClientCards($data)
	{
		return $this->request($data, "eCom_api/client_identification/card/{$data['mid']}/{$data['client_id']}", 'DELETE');
	}

	public function getClientCards($data)
	{
		return $this->request($data, "eCom_api/client_identification/{$data['mid']}/{$data['client_id']}", 'GET');
	}

	public function purchase($data)
	{
		return $this->request($data, '/eCom_api/finOperate/purchase/without3DS');
	}

	public function capturePayment($data)
	{
		$data['Action'] = 'Capture';
		return self::paymentAction($data);
	}

	public function reversePayment($data)
	{
		$data['Action'] = 'Reverse';
		return self::paymentAction($data);
	}

	public function refundPayment($data)
	{
		$data['Action'] = 'Refund';
		return self::paymentAction($data);
	}

	public function paymentAction($data)
	{
		$data['Version']                  = '1.0.0';
		$data['SignatureMethod']          = 'SHA256';
		$data['PurchaseCurrency']         = '643';
		$data['PurchaseCurrencyExponent'] = '2';
		if (empty($data['MerID'])) {
			throw new EmptyPropertyException("Empty MerID", 0, "MerID");
		}
		if (empty($data['AcqID'])) {
			throw new EmptyPropertyException("Empty AcqID", 0, "AcqID");
		}
		if (empty($data['PurchaseAmt'])) {
			throw new EmptyPropertyException("Empty PurchaseAmt", 0, "PurchaseAmt");
		}
		if (empty($data['OrderID'])) {
			throw new EmptyPropertyException("Empty OrderID", 0, "OrderID");
		}
		if (empty($data['AuthorizationNumber'])) {
			throw new EmptyPropertyException("Empty AuthorizationNumber", 0, "AuthorizationNumber");
		}
		if (empty($data['Amount'])) {
			throw new EmptyPropertyException("Empty Amount", 0, "Amount");
		}
		if (empty($data['Signature'])) {
			throw new EmptyPropertyException("Empty Signature", 0, "Signature");
		}

		$result = $this->request($data, 'finoperate/dofinancialoperationservlet');
		$result = $this->parseOperationResponse($result);
		return $result;
	}

	public function parseOperationResponse($response)
	{
		if (!$response) {
			return false;
		}
		$decoded = htmlspecialchars_decode($response);
		parse_str($decoded, $result);
		return $result;
	}

	public function registerReceipt($merchantId, $receipt, $signature)
	{

		$action  = 'OnlineReceipt/1/' . $merchantId . '/receipt';
		$data    = json_encode($receipt);
		$headers = [
			'Signature: ' . $signature,
			'Content-Type: application/json',
		];
		return $this->request($data, $action, 'POST', $headers);
	}

	private function getBaseUrl()
	{
		return $this->testMode ? self::TEST_URL : self::PROD_URL;
	}

	private function request($data, $action, $method = 'POST', $headers = [])
	{
		if (!function_exists("curl_init")) {
			throw new ExtensionNotFoundException("curl");
		}

		$ch  = curl_init();
		$url = $this->getBaseUrl() . $action;

		if ($method == 'GET') {
			$url .= "?" . http_build_query($data);
			$data = [];
		}

		$curlOptions = [
			CURLOPT_URL            => $url,
			CURLOPT_RETURNTRANSFER => 1,
			CURLOPT_VERBOSE        => 0,
			CURLOPT_USERAGENT      => 'MKB PHP Sdk',
			CURLOPT_HTTPHEADER     => $headers,
			CURLOPT_SSL_VERIFYHOST => 0,
			CURLOPT_SSL_VERIFYPEER => 0,
		];

		if ($method == 'POST') {
			$curlOptions[CURLOPT_POST] = true;
		}

		if ($method == 'DELETE') {
			$curlOptions[CURLOPT_CUSTOMREQUEST] = 'DELETE';
		}

		if (!empty($data)) {
			$curlOptions[CURLOPT_POSTFIELDS] = is_array($data) ? http_build_query($data) : $data;
		}

		curl_setopt_array($ch, $curlOptions);
		$response = curl_exec($ch);
		$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		if ($code !== 200) {
			throw new ServletResponseException("Servlet responded with {$code} code", $code, $response);
		}
		curl_close($ch);

		return $response;
	}
}
